import {Component, OnInit, ChangeDetectorRef, Renderer2, ElementRef, ViewChild, inject, QueryList, ViewChildren, NgModule,
  AfterViewInit, contentChild} from '@angular/core';
import {FormBuilder, FormControl, FormGroup, FormsModule, ReactiveFormsModule, Validators} from '@angular/forms';
import {MatIconModule} from '@angular/material/icon';
import {MatDividerModule} from '@angular/material/divider';
import {MatButtonModule} from '@angular/material/button';
import {provideNativeDateAdapter,  MatNativeDateModule, MAT_DATE_FORMATS, MAT_DATE_LOCALE, DateAdapter } from '@angular/material/core';
import {DateRange, MatDatepickerModule, MatDateRangeInput   } from '@angular/material/datepicker';
import {NgxMatTimepickerModule } from '@alexfriesen/ngx-mat-timepicker';
import {MatFormFieldModule} from '@angular/material/form-field';
import {MatInputModule} from '@angular/material/input';
import {MatSelectModule} from '@angular/material/select';
import {MatButtonToggleModule} from '@angular/material/button-toggle';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {ConformidadHelper} from "./helper/conformidad.helper";
import {ConformidadService} from "../../service/conformidad_service/conformidad.service";
import {NgClass, NgForOf, NgIf, DatePipe, AsyncPipe, NgStyle} from "@angular/common";
import {MatCard, MatCardContent, MatCardFooter, MatCardHeader, MatCardTitle , MatCardModule} from '@angular/material/card';
import {MatAutocompleteModule} from '@angular/material/autocomplete';
import {MatTooltipModule} from '@angular/material/tooltip';
import {MatDialog, MatDialogModule} from '@angular/material/dialog';
import {modalRequerimientoComponent} from "./modalRequerimiento/modalRequerimiento.componente";
import {modalFirmaComponent} from "./modalFirma/modalFirma.componente";
import {MatTableModule} from '@angular/material/table';
import { MatCell, MatCellDef, MatColumnDef, MatHeaderCell, MatHeaderRow, MatHeaderRowDef, MatRow, MatRowDef, MatTable } from "@angular/material/table";
import { SoloNumerosDirective } from '../../service/directivas_service/soloNumeros/solo-numeros.directive'; // Importa la directiva
import { MY_DATE_FORMATS } from "../../service/directivas_service/formatoFecha/date-format"; // Asegúrate de ajustar la ruta
import { MatSnackBar, MatSnackBarHorizontalPosition, MatSnackBarVerticalPosition,} from '@angular/material/snack-bar';
import { ConfirmarEliminacionComponent  } from './modalEliminarPasaje/eliminarPasaje.componente';
import { modalEvidenciaComponent } from "./modalEvidencia/modalEvidencia.componente";
import { truncarCaracteresDirective } from "../../service/directivas_service/restriccionCaracteres/truncarCaracteres.directive";
import { ordenSalidaComponent } from "../ordensalida/ordensalida.componente";
import {elementSelectors} from "@angular/cdk/schematics";
import { CustomDateAdapter } from '../../service/directivas_service/formatoFecha/custom-date'; // Archivo personalizado
import Notiflix from 'notiflix';
import {environment} from "../../../environment/env";

interface tiposTrabajo {
  codtipo: string;
  desctipo: string;
}

interface lugaresTrabajo {
  value : string;
  viewValue : string;
}

interface interRequerimientos {
  id: number;
  titulo: string;
  codigo: string
}

interface interPersonal {
  index : number;
  id : number ;
  nombre : string ;
}

interface interFile{
  tipoEvidencia: number;
  file: File;
}

@Component({
  imports: [
    SoloNumerosDirective,
    MatDatepickerModule,
    NgxMatTimepickerModule,
    MatNativeDateModule,
    MatFormFieldModule,
    FormsModule,
    ReactiveFormsModule,
    MatSelectModule,
    MatInputModule,
    MatButtonToggleModule,
    MatCheckboxModule,
    NgForOf,
    MatCard,
    MatCardContent,
    MatCardFooter,
    MatCardHeader,
    MatCardTitle,
    MatCardModule,
    MatButtonModule,
    MatDividerModule,
    MatIconModule,
    MatAutocompleteModule,
    AsyncPipe,
    MatTooltipModule,
    NgClass,
    MatCell,
    MatCellDef,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderRow,
    MatHeaderRowDef,
    MatRow,
    MatRowDef,
    MatTable,
    MatTableModule,
    NgIf,
    NgStyle,
    truncarCaracteresDirective,
    ordenSalidaComponent,
  ],
  providers: [
    provideNativeDateAdapter(),
    DatePipe,
    { provide: MAT_DATE_LOCALE, useValue: 'es-ES' }, // Configurar el idioma español
    { provide: DateAdapter, useClass: CustomDateAdapter } // Adaptador personalizado
  ],
  selector: 'app-conformidad',
  standalone: true,
  templateUrl: './conformidad.componente.html',
  styleUrls: ['./conformidad.componente.css'],
  //changeDetection: ChangeDetectionStrategy.OnPush,
})

export class ConformidadComponent implements AfterViewInit {
  private barraAlerta = inject(MatSnackBar);

  // VIEWCHILD
  @ViewChild('labelAdvertencia', { static: false }) labelAdvertencia!: ElementRef;
  @ViewChild('inputBusqueda', { static: false }) inputBusqueda!: ElementRef;
  @ViewChildren('fileEditar1') fileInputs1!: QueryList<ElementRef> ;
  @ViewChildren('fileEditar2') fileInputs2!: QueryList<ElementRef> ;
  @ViewChild(ordenSalidaComponent) ordenSalidaComponent!: ordenSalidaComponent;

  // VIEWCHILD

  //INICIALIZACION DE VARIABLES UTILIZABLES

  opcionEnvironmnet : boolean = environment.isProduction;
  personal_list: any = [];
  sede_list: any = [];
  cargosU_list : any = [];
  zona_list: any = [];
  unidades_list : any = [];
  requerimientosFiltrados: any[] = [];
  listSedesAtencion : lugaresTrabajo[] = [];
  listRequerimientos: interRequerimientos[] = [];
  listTipoTrabajo: tiposTrabajo[] = [];
  listPersonal : interPersonal[] = [{ index: 0 , id: 0 , nombre: '' }];
  idBotones: number = 0;
  labelText: string = ''; // Texto que se muestra inicialmente
  isFading: boolean = false; // Controla si la animación de desvanecimiento debe activarse
  personalText : string = '';
  personalFading : boolean = false ;
  mostrarHoras : boolean = true ;
  obtenerFirma :boolean = false;

  alertaPosicionHorizontal: MatSnackBarHorizontalPosition = 'center';
  alertaPosicionVertical: MatSnackBarVerticalPosition = 'bottom';
  alertaDuracion : number = 3;

  archivosEvidencia : interFile[] = [];
  archivosProblema : interFile[] = [];
  archivosSolucion : interFile[] = [];
  //INICIALIZACION DE VARIABLES UTILIZABLES

  // GRUPO DE FORMULARIO

  datos_personal = new FormGroup({
     tecnicoPersonal0 : new FormControl(null ,Validators.required),
     areaPersonal0 : new FormControl<string>('' ,Validators.required),
     zonaPersonal : new FormControl(null,Validators.required),
     tipoCargoPersonal : new FormControl<string>('' ,Validators.required),
     sedePersonal : new FormControl(null,Validators.required),
     minFechaTrabajo : new FormControl<Date>(new Date(), Validators.required),
     maxFechaTrabajo : new FormControl<Date>(new Date(), Validators.required),
     minHoraTrabajo : new FormControl<string | null>(''),
     maxHoraTrabajo : new FormControl<string | null>(''),
  });

  datos_servicio = new FormGroup({
    busquedaReq : new FormControl<string | null>(null ,[Validators.maxLength(12)]),
    descripcionServicio : new FormControl<string | null>(null ,Validators.required),
    causaServicio : new FormControl<string | null>(null ,Validators.required),
  });

  datos_trabajo = new FormGroup({
    tipoTrabajo : new FormControl<string>('1' ,Validators.required),
    descripcionTrabajo : new FormControl<string>('' ,Validators.required),
    avanceTrabajo : new FormControl('' ,[Validators.required, Validators.min(1),Validators.max(100)]),
  });

  existeMaterialS : FormGroup = new FormGroup({
    existeMaterial : new FormControl(null ,Validators.required),
  });

  // GROUPO DE FORMULARIO

  constructor(
    private conformidadHelper: ConformidadHelper,
    private ConformidadService : ConformidadService,
    private renderer: Renderer2,
    private detectorChange : ChangeDetectorRef,
    public dialog: MatDialog,
    private datePipe: DatePipe,
    private fb: FormBuilder,
    private snackBar: MatSnackBar
  ) {
    this.requerimientosFiltrados = [];
    renderer.setStyle(document.body, 'background', 'var(--light-theme-bg)');
  }

  //ACCESOS RAPIDOS A GRUPOS DE FORMULARIOS

  agregarFormControl(formGroup: FormGroup, formControlName: string, formControl: FormControl = new FormControl()): any {
    return formGroup.addControl(formControlName, formControl);
  }

  obtenerFormControl(formGroup: FormGroup, formControlName: string): any {
    return formGroup.get(formControlName);
  }

  removerFormControl(formGroup: FormGroup, formControlName: string): any {
    return formGroup.removeControl(formControlName);
  }

  //ACCESOS RAPIDOS A GRUPOS DE FORMULARIOS

  formatoFecha(fecha: Date): string {
    return this.datePipe.transform(fecha, 'dd/MM/yyyy') || '';
  }

  formatoFechaDate(fecha: Date): string {
    return this.datePipe.transform(fecha, 'yyyy/MM/dd') || '';
  }

  formatoFechaRegistro(fecha: Date){
    return this.datePipe.transform(fecha, 'yyyy-MM-dd') || '';
  }


  async ngAfterViewInit(): Promise<void> {

    const [
      personal,
      sede,
      tipoTrabajo,
      cargosUnidos,
      unidades
    ] = await Promise.all([
      this.conformidadHelper.cargarPersonal(),
      this.conformidadHelper.cargarSedes(),
      this.conformidadHelper.gestionConformidad({opcion: 9}),
      this.conformidadHelper.cargarCargosU(),
      this.conformidadHelper.listadoGeneralHorizon({accion : 5})
    ]);

    //this.personal_list = personal;
    this.sede_list = sede;
    this.cargosU_list = cargosUnidos;
    this.zona_list = [{ value: 1, viewValue: 'NORTE' },{ value: 2, viewValue: 'SUR' },{ value: 3, viewValue: 'ESTE' },{ value: 4, viewValue: 'CENTRO' }];
    this.unidades_list = unidades;
    this.listRequerimientos = [];
    const jsonTipoTrabajo = JSON.parse(tipoTrabajo);
    this.listTipoTrabajo = jsonTipoTrabajo.data;

    this.obtenerFormControl(this.datos_personal,'zonaPersonal')?.valueChanges.subscribe((value : any) => {
      this.listSedesAtencion = this.sede_list.filter((item: any) => item.zona === value).map((item: any) => ({ value: item.id_local, viewValue: item.deslocal }));
    });

    this.obtenerFormControl(this.datos_personal,'tipoCargoPersonal')?.valueChanges.subscribe((value : any) => {
      const cargos = value.split(',').map((item: any) => Number(item));
      this.personal_list = personal.filter((item: any) => cargos.includes(item.codcargo));
    });

    this.datos_personal.get('maxFechaTrabajo')?.valueChanges.subscribe((value) => {
      let minFecha : any = this.datos_personal.get('minFechaTrabajo')?.value;
      let maxFecha : any = value;

      if( minFecha && maxFecha ){

        const min = this.formatoFechaDate(new Date(minFecha));
        const max = this.formatoFechaDate(new Date(maxFecha));

        if(min == max){
          this.mostrarHoras = true;
        }else{
          this.mostrarHoras = false;
        }

        this.datos_personal.get('minHoraTrabajo')?.setValue('00:00');
        this.datos_personal.get('maxHoraTrabajo')?.setValue('00:00');

      }
    });
  }

  //AL SELECCIONAR UN REQUERIMIENTO
  onEventSeleccion(event : any){
    const valor = event.option.value;

    if(this.listRequerimientos.find((item) => item.codigo === valor)){
      this.labelText = 'Ya existe el requerimiento ' + valor + ' en el listado'; // Cambia el texto
      this.isFading = false; // Resetea el estado de opacidad para mostrar el texto

      // Después de 2 segundos, comienza a desvanecer el texto
      setTimeout(() => {
        this.isFading = true;
      }, 2000); // 2000 ms = 2 segundos

    }else{
      this.listRequerimientos.push({ titulo: `RQ N°${valor}`, id: this.idBotones++ , codigo: valor });
      this.labelText = '';
    }
    this.datos_servicio.get('busquedaReq')?.setValue('');
    this.inputBusqueda.nativeElement.blur();
    this.detectorChange.detectChanges();
    this.requerimientosFiltrados = [];

  }

  //AL SELECCIONAR INFORMACION DE UN REQUERIMIENTO
  async infoRequerimiento(valor: any){
    const requerimientoInfo = await this.conformidadHelper.listadoGeneralHorizon({accion: 3, nombreValor: valor, sede: ''});
    const dialogRef = this.dialog.open(modalRequerimientoComponent,{
      width: '100%',
      maxWidth: '90%',
      minWidth: '50%',
      height: '50%',
      data: { requerimientoInfo }
    });
    dialogRef.afterClosed().subscribe(result => {
      console.log(`Dialog result: ${result}`);
    });
  }

  //ELIMINAR REQUERIMIENTO SELECCIONADO
  deleteRequerimiento(valor : any){
    this.listRequerimientos = this.listRequerimientos.filter((item) => item.id !== valor);
  }

  //AGREGAR UN TECNICO
  onEventoAgregarPersonal(){
    this.agregarFormControl(this.datos_personal,'tecnicoPersonal'+this.listPersonal.length,new FormControl(null,Validators.required));
    this.agregarFormControl(this.datos_personal,'areaPersonal'+this.listPersonal.length,new FormControl<string>('' ,Validators.required));
    this.listPersonal.push({ index: this.listPersonal.length , id: 0 , nombre: '' }); // Agregar uno más

  }
  //QUITAR UN TECNICP
  onEventoQuitarPersonal(){

    if(this.listPersonal.length === 1){
      this.personalText = 'Debe de haber un trabajador como minimo'; // Cambia el texto
      this.personalFading = false; // Resetea el estado de opacidad para mostrar el texto

      // Después de 2 segundos, comienza a desvanecer el texto
      setTimeout(() => {
        this.personalFading = true;
      }, 2000); // 2000 ms = 2 segundos

    }else{
      this.listPersonal.pop(); // Eliminar el ultimo elemento del array
      this.removerFormControl(this.datos_personal,'tecnicoPersonal'+this.listPersonal.length);
      this.removerFormControl(this.datos_personal,'areaPersonal'+this.listPersonal.length);
    }

  }
  //AL SELECCIONAR UN TECNICO, MOSTRAR AUTOMATICAMENTE SU AREA
  onEventoSeleccionTecnico(event: any, elemento : any) { // IMPRIMER EL ÁREA DEL TÉCNICO AL SELECCIONARLO

    const codigoPer = event.value;
    const personal_list = this.personal_list;
    const personal_datos = personal_list.find((item: any) => item.codper === codigoPer);
    const areaControl = this.datos_personal.get('areaPersonal'+elemento.index);
    const validarRepetido = this.listPersonal.find((item: any) => item.id === codigoPer && item.index != elemento.index);

    if(validarRepetido){

      this.datos_personal.get('tecnicoPersonal'+elemento.index)?.setValue(null);
      this.personalText = 'Personal ya seleccionado'; // Cambia el texto
      this.personalFading = false; // Resetea el estado de opacidad para mostrar el texto

      // Después de 2 segundos, comienza a desvanecer el texto
      setTimeout(() => {
        this.personalFading = true;
      }, 2000); // 2000 ms = 2 segundos

    }else{

      this.personalText = '';
      this.listPersonal[elemento.index].id = codigoPer;
      this.listPersonal[elemento.index].nombre = personal_datos.nombre;

      if(Object.keys(personal_datos).length  > 0){
        areaControl?.setValue(personal_datos.descargo);
      }else{
        areaControl?.setValue('Sin Cargo');
      }
      areaControl?.updateValueAndValidity();

    }

  }

  //REALIZAR UNA BUSQUEDA DE REQUERIMIENTO
  async onBusquedaRequerimiento(event: any) {
    //const codigoSede = this.obtenerFormControl(this.datos_personal,'sedePersonal')?.value;
    const busquedaReq = event.target.value;

    const jsonParametrosReq = {
      accion : 2,
      nombreValor : busquedaReq,
      sede : ''
    };

    const requerimiento_list = await this.conformidadHelper.listadoGeneralHorizon(jsonParametrosReq) ;
    const mapeoResultado = requerimiento_list.map((item: any) => ({
      codigo: item.CODIGONUM,
      serie : item.CSERIE_DOC,
      numero: item.CODIGONUM
    }));

    this.requerimientosFiltrados = mapeoResultado;
  }

  onEventoSubirArchivo(event: any, tipo : number = 1) {

    const archivosSeleccionados: FileList = event.target.files;
    const nuevosArchivos = Array.from(archivosSeleccionados).filter(file => file.type === 'image/png' || file.type === 'image/jpeg');
    nuevosArchivos.forEach( async (file) => {
      if (!this.archivosEvidencia.find((item) => item.file.name === file.name)) {
        this.archivosEvidencia.push({tipoEvidencia: tipo, file: file});
      }
    });

    this.archivosProblema = this.archivosEvidencia.filter((item) => item.tipoEvidencia === 1);
    this.archivosSolucion = this.archivosEvidencia.filter((item) => item.tipoEvidencia === 2);


  }

  seleccionarInput(archivo : any, tipo : number){

    if(tipo === 1){
      const index = this.archivosProblema.findIndex((item) => item.file.name === archivo.file.name);
      const input = this.fileInputs1.toArray()[index];
      input.nativeElement.click(); // Simula el clic en el input file correspondiente
    }else if(tipo === 2){
      const index = this.archivosSolucion.findIndex((item) => item.file.name === archivo.file.name);
      const input = this.fileInputs2.toArray()[index];
      input.nativeElement.click(); // Simula el clic en el input file correspondiente
    }

  }

  async gestionArchivos(index: number, accion : number, file : any = [], tipo : number = 1){

    if(accion === 1){
      const fileReader = new FileReader();
      fileReader.onload = () => {
        this.dialog.open(modalEvidenciaComponent, {
          data: {
            imageUrl: fileReader.result // Pasar la imagen al modal
          }
        });
      };
      fileReader.readAsDataURL(file); // Convertir el archivo a URL para mostrarlo

    }else if(accion === 2){

      const archivoSeleccionado = file.target.files[0]; // Obteniendo el File
      //OBTENER EL ARCHIVO FILE Y UBICARLO EN EL ARRAY DE archivosEP EN EL MISMO INDEX
      if(archivoSeleccionado) {
        //COMPRAR EL NOMBRE DEL ARCHIVOS CON TODA LA LISTA EXISTENTE PARA EVITAR REPETICIONES
        if(!this.archivosEvidencia.find((item) => item.file.name === archivoSeleccionado.name && item.tipoEvidencia === tipo)){
          this.archivosEvidencia[index].file = archivoSeleccionado;
        }else{
          this.barraAlerta.open( 'ARCHIVO YA SE ENCUENTRA EN LA LISTA' ,  '' , {
            horizontalPosition: this.alertaPosicionHorizontal,
            verticalPosition: this.alertaPosicionVertical,
            duration: this.alertaDuracion * 1000
          });
        }
      }

    }else{
      this.archivosEvidencia.splice(index, 1); //  Eliminar el archivo del array de archivosEP
    }

  }

  async gestionConformidad() {

    //Obtener todos los datos del form Group datos_personal
    const datosPersonal = this.datos_personal.getRawValue();
    const personal = this.listPersonal;
    //const datosPasajes = this.dataSource; // Obtener todos los datos de la tabla de pasajes
    const datosServicio = this.datos_servicio.getRawValue();
    const requerimientoRelacionados = this.listRequerimientos;
    const datosTrabajo = this.datos_trabajo.getRawValue();
    const existeMaterial = parseInt(this.existeMaterialS.get('existeMaterial')?.value) || 0;
    const datosMaterial = this.ordenSalidaComponent?.dataSource.length > 0 ? this.ordenSalidaComponent?.dataSource : [];
    const evidenciaMaterial = this.ordenSalidaComponent?.archivosMaterial.length > 0 ? this.ordenSalidaComponent?.archivosMaterial : [];
    const archivosEvidencias = this.archivosEvidencia;
    let contadorEP = 1;
    let contadorES = 1;
    let constadorATS = 1;
    let contadorFirma = 1;
    let contadorMaterialSobrante = 1;
    let contadorExtras = 1;
    const ordenSalidaMaterial = this.ordenSalidaComponent?.datos_materialSobrante?.get('numeroOrden')?.value ? this.ordenSalidaComponent?.datos_materialSobrante?.get('numeroOrden')?.value : '';
    const descripcionMaterial = this.ordenSalidaComponent?.datos_materialSobrante?.get('descMaterial')?.value ? this.ordenSalidaComponent?.datos_materialSobrante?.get('descMaterial')?.value : '';

    Notiflix.Loading.standard('Cargando...');

    if(evidenciaMaterial.length > 0){
      archivosEvidencias.push(...evidenciaMaterial);
    }

    if (!this.validarDatosPersonales(datosPersonal)) {
      this.mostrarAlerta('VALIDAR DATOS PERSONALES');
      return;
    }

    if(!this.validarPersonal(personal)){
      this.mostrarAlerta('VALIDAR PERSONAL ASIGNADO');
      return;
    }

    if(!this.validarArchivos(archivosEvidencias)){
      return;
    };

    if (!this.validarMaterialSobrante(existeMaterial, this.ordenSalidaComponent?.dataSource)) {
      this.mostrarAlerta('VALIDAR MATERIAL SOBRANTE');
      return;
    }

    if(!this.validarDatosTrabajo(datosTrabajo)){
      this.mostrarAlerta('VALIDAR DATOS DE TRABAJO');
      return;
    }

    const registroConformidad = await this.conformidadHelper.gestionConformidad({
      opcion: 1,
      codlocal: datosPersonal.sedePersonal + '',
      fechainicio: this.formatoFechaRegistro(datosPersonal.minFechaTrabajo || new Date() ) ,
      fechafin: this.formatoFechaRegistro(datosPersonal.maxFechaTrabajo || new Date() ) ,
      descproblema: datosServicio.descripcionServicio,
      desccausa: datosServicio.causaServicio,
      codtipotrabajo: parseInt(datosTrabajo.tipoTrabajo || '1'),
      desctrabajo: datosTrabajo.descripcionTrabajo,
      minhoratrabajo : datosPersonal.minHoraTrabajo,
      maxhoratrabajo : datosPersonal.maxHoraTrabajo,
      avance : datosTrabajo.avanceTrabajo,
    });

    const respuestaConformidad : any = JSON.parse(registroConformidad);

    if(respuestaConformidad.status){ // DATOS DE CONFORMIDAD VALIDADOS Y REGISTRADOS

      let codigoConformidad = respuestaConformidad.data.codconformidad;

      const registroCarpeta = await this.conformidadHelper.creacionCarpeta({codconformidad: codigoConformidad});

      if (registroCarpeta.status) {
        //UNIR LOS DATOS archivosEP Y  archivosES EN UN SOLO ARRAY

        const registroArchivos = archivosEvidencias.map((item) => {
          return {
            codconformidad: codigoConformidad,
            archivo: item.file,
            idCarpeta: registroCarpeta.idCarpeta,
            tipoEvidencia: item.tipoEvidencia,
            nombreArchivo: (item.tipoEvidencia === 1) ? 'EVIDENCIA_PROBLEMA_' + codigoConformidad + "_" + (contadorEP++) :
              (item.tipoEvidencia === 2) ? 'EVIDENCIA_SOLUCION_' + codigoConformidad + "_" + (contadorES++) :
                (item.tipoEvidencia === 3) ? 'EVIDENCIA_ATS_' + codigoConformidad + "_" + (constadorATS++) :
                  (item.tipoEvidencia === 4) ? 'EVIDENCIA_FIRMA_' + codigoConformidad + "_" + (contadorFirma++) :
                    (item.tipoEvidencia === 5) ? 'EVIDENCIA_MATERIALSOBRANTE_' + codigoConformidad + "_" + (contadorMaterialSobrante++) : 'EVIDENCIA_DATOEXTRA_' + codigoConformidad + "_" + (contadorExtras++)
          };
        });

        this.ConformidadService.progresoSubida.next(0); // Reiniciar progreso

        const resultados: {
          status: boolean,
          mensaje: string
        } = await this.conformidadHelper.subidaEvidencia3(registroArchivos);

        if (resultados.status) { // REGISTROS DE EVIDENCIAS CONFIRMADAS Y REGISTRADAS

          Notiflix.Loading.change(`Registro de Personal...`);

          let statusPersonal = true;

          const datosPersonalTecnico = personal.map((item) => {
            return {
              opcion : 8,
              codconformidad : codigoConformidad,
              codper : item.id,
            }
          });

          for (let i=0; i < datosPersonalTecnico.length; i++){
            const registroRequerimiento = await this.conformidadHelper.gestionConformidad(datosPersonalTecnico[i]);
            const respuestaRequerimiento = JSON.parse(registroRequerimiento);
            if(!respuestaRequerimiento.status) {
              statusPersonal = false;
              break;
            }
          }

          if(statusPersonal) { // REGISTRO DE PERSONAL CONFIRMADO Y REGISTRADO

            let statusRequerimientos = true;

            if (requerimientoRelacionados.length > 0) {

              Notiflix.Loading.change(`Registro de Requerimientos...`);

              const datosRequerimientos = requerimientoRelacionados.map((item) => {
                return {
                  opcion: 3,
                  codconformidad: codigoConformidad,
                  codrequerimiento: parseInt(item.codigo)
                };
              });

              for (let i = 0; i < datosRequerimientos.length; i++) {
                const registroRequerimiento = await this.conformidadHelper.gestionConformidad(datosRequerimientos[i]);
                const respuestaRequerimiento = JSON.parse(registroRequerimiento);
                if (!respuestaRequerimiento.status) {
                  statusRequerimientos = false;
                  break;
                }
              }
            }else{
              statusRequerimientos = true;
            }

            if (statusRequerimientos) { // REGISTRO DE REQUERIMIENTOS VALIDADOS Y REGISTRADOS

              // UNA VEZ REGISTRADO TODOS LOS DATOS DE LA CONFORMIDAD, SE PROCEDE A REGISTRAR LA ORDEN DE SALIDA RELACIONADA.

              let statusOrdenSalida = true;

              if (existeMaterial === 1) {

                Notiflix.Loading.change(`Registro de Orden de Salida...`);

                const registroOrdenSalida = await this.conformidadHelper.gestionOrdenSalida({
                  accion: 1,
                  codlocal: datosPersonal.sedePersonal + '',
                  fechaRegistro: this.formatoFechaRegistro(new Date()),
                  descripcion: descripcionMaterial,
                  numordensalida : ordenSalidaMaterial
                });

                const respuestaOrdenSalida: any = JSON.parse(registroOrdenSalida);

                if (respuestaOrdenSalida.status) { // REGISTRO DE ORDEN DE SALIDA CONFIRMADO Y VALIDADO

                  const datosMaterialSobrante = datosMaterial.map((item) => {
                    return {
                      accion: 2,
                      codOrdenSalida: respuestaOrdenSalida.data.codordensalida,
                      codArticulo: item.codigo,
                      nombreArticulo: item.nombre,
                      unidad: item.unidad,
                      cantidad: item.cantidad,
                      estado: item.estado
                    };
                  });
                  for (let i = 0; i < datosMaterial.length; i++) {
                    const registroMaterialS = await this.conformidadHelper.gestionOrdenSalida(datosMaterialSobrante[i]);
                    const respuestaregistroMaterialS = JSON.parse(registroMaterialS);
                    if (!respuestaregistroMaterialS.status) {
                      statusOrdenSalida = false;
                      break;
                    }
                  }
                  if (statusOrdenSalida) { // UNION DE LA ORDEN DE SALIDA CON LA CONFORMIDAD

                    const relacionarOrdenConformidad = await this.conformidadHelper.gestionConformidad({
                      opcion: 6,
                      codconformidad: codigoConformidad,
                      codordensalida: respuestaOrdenSalida.data.codordensalida
                    });

                    const respuestarelacionarOrdenConformidad = JSON.parse(relacionarOrdenConformidad);
                    if (!respuestarelacionarOrdenConformidad.status) {
                      statusOrdenSalida = false;
                    }
                  }
                } else {
                  statusOrdenSalida = false; //this.mostrarAlerta('ERROR AL REGISTRAR LOS ORDEN DE SALIDA');
                }
              } else {
                statusOrdenSalida = true;
              }

              if (statusOrdenSalida) {

                Notiflix.Loading.remove();
                Notiflix.Notify.success('ÉXITO DE CONFORMIDAD ', {
                  position: 'center-center', // Notificación en el centro de la pantalla
                  timeout: 1000, // 1 segundo
                  width: '400px',
                  fontSize: '24px',
                  fontAwesomeIconSize: '34px',
                });

                // Espera 1 segundo y recarga la página
                setTimeout(() => {
                  window.location.reload();
                }, 1000);
              }else {
                this.mostrarAlertaError('FALLO DE REGISTRO','ERROR AL REGISTRAR ORDEN DE SALIDA');
              }
            } else {
              this.mostrarAlertaError('FALLO DE REGISTRO','ERROR AL REGISTRAR LOS REQUERIMIENTOS');
            }
          }else{
            this.mostrarAlertaError('FALLO DE REGISTRO','ERROR AL REGISTRAR EL PERSONAL');
          }
        } else {
          this.mostrarAlertaError('ERROR DE SUBIDA DE EVIDENCIA' , resultados.mensaje );
          //Agregar el eliminar registro y eliminar Documento.
          await this.conformidadHelper.eliminarDocumento({
            opcion: 1,
            codconformidad: codigoConformidad
          });
        }
      } else{
        this.mostrarAlertaError('ERROR DE SUBIDA DE EVIDENCIA','ERROR AL CREAR CARPETA DE EVIDENCIAS');
      }
    }else{
      this.mostrarAlertaError('FALLO DE REGISTRO','ERROR AL REGISTRAR LA CONFORMIDAD');
    }

  }

  validarDatosPersonales(datosPersonal: any): boolean {

    const min = this.formatoFechaDate(new Date(datosPersonal?.minFechaTrabajo));
    const max = this.formatoFechaDate(new Date(datosPersonal?.maxFechaTrabajo));
    let validarHoras;
    if(min == max){
      validarHoras = datosPersonal?.minHoraTrabajo && datosPersonal?.maxHoraTrabajo;
    }else{
      validarHoras = true;
    }

    return datosPersonal?.sedePersonal &&
      datosPersonal?.maxFechaTrabajo &&
      datosPersonal?.minFechaTrabajo && validarHoras ;
  }


  private validarDatosTrabajo(datosTrabajo: any) : boolean{

    const TipoTrabajo = datosTrabajo.tipoTrabajo;
    const Avance = datosTrabajo.avanceTrabajo;

    return TipoTrabajo !== null && Avance !== null && Avance <= 100 && Avance >= 1;
  }

  validarPersonal(personal: any[]): boolean {
    return personal.length > 0 && personal.every((item) => item.id != '0' && item.nombre != '');
  }

  validarArchivos(archivosEvidencia : any[]): boolean {

    let advertencia = '';
    if(archivosEvidencia.find((item) => item.tipoEvidencia === 1) === undefined){
      advertencia += `REQUIERE SUBIR EVIDENCIA DE PROBLEMA`;
    }
    else if(archivosEvidencia.find((item) => item.tipoEvidencia === 2) === undefined){
      advertencia += `REQUIERE SUBIR EVIDENCIA DE SOLUCION`;
    }
    else if(archivosEvidencia.find((item) => item.tipoEvidencia === 3) === undefined){
      advertencia += `REQUIERE SUBIR EVIDENCIA DE ATS`;
    }
    /*else if(archivosEvidencia.find((item) => item.tipoEvidencia === 4) === undefined){
      advertencia += `REQUIERE SUBIR EVIDENCIA DE FIRMA`;
    }*/

    if(advertencia !== ''){
      this.mostrarAlerta(advertencia);
    }

    return (advertencia === '');//Debe existir por lo menos un archivo de evidencia de problema y uno de solucion
  }

  validarMaterialSobrante(existeMaterial: number, dataSource: any[]): boolean {
    return (existeMaterial === 1 && dataSource.length > 0) || existeMaterial === 0;
  }

  mostrarAlerta(mensaje: string): void {
    Notiflix.Loading.remove();
    this.barraAlerta.open(mensaje, '', {
      horizontalPosition: this.alertaPosicionHorizontal,
      verticalPosition: this.alertaPosicionVertical,
      duration: this.alertaDuracion * 1000,
      panelClass: ['custom-snackbar'],
    });
  }

  mostrarAlertaError(titulo: string , mensaje: string): void {
    Notiflix.Loading.remove();
    Notiflix.Report.failure(
      titulo,
      mensaje,
      "Cerrar"
    );
  }

  mostrarFirma(valor: any){
    const dialogRef = this.dialog.open(modalFirmaComponent,{

    });
    dialogRef.afterClosed().subscribe(result => {
      if(result !== 0 && result !== undefined){
        if(this.obtenerFirma){

        }else{
          const Archivo = this.blobToFile(result.blob, result.nombre);
          this.archivosEvidencia.push({tipoEvidencia: 4, file: Archivo});
        }
        this.obtenerFirma = true;

      }else{
        this.obtenerFirma = false;
        this.mostrarAlerta('FIRMA NO REGISTRADA');
      }
    });

  }

  blobToFile(blob: Blob, fileName: string, mimeType?: string): File {
    return new File([blob], fileName, { type: mimeType || blob.type });
  }

  fileToBase64(file: File): Promise<string> {
    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      reader.readAsDataURL(file);
      reader.onload = () => resolve(reader.result as string);
      reader.onerror = (error) => reject(error);
    });
  }

}

