import {Component, Inject, EventEmitter, ViewChild, NgModule, AfterViewInit, Output, ElementRef} from '@angular/core';
import { MatDialogActions, MatDialogClose, MatDialogContent, MatDialogTitle, MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import {MatButtonModule} from '@angular/material/button';
import {NgIf} from "@angular/common";
import SignaturePad from 'signature_pad';

@Component({
  selector: 'app-file-preview-dialog',
  imports: [MatButtonModule,
    MatDialogActions,
    MatDialogClose,
    MatDialogContent,
    MatDialogTitle,
    NgIf,
  ],
  standalone: true,
  styleUrls: ['./modalFirma.componente.css'],
  template:`
    <h1 mat-dialog-title>Firma de conformidad</h1>

        <div #division class="singpad" align="center">
          <!-- <signature-pad #signature (drawStart)="drawStart($event)" (drawEnd)="drawComplete($event)"></signature-pad> -->
          <canvas
            #canvas
            (touchstart)="drawStart($event)"
            (touchmove)="drawComplete($event)"
          ></canvas>
        </div>

    <mat-dialog-actions align="center">
      <button mat-button (click)="onClear()">Limpiar</button>
      <button mat-button (click)="onCancelar()">Cancelar</button>
      <button mat-button (click)="onGuardar()" cdkFocusInitial>Guardar</button>
    </mat-dialog-actions>
  `
})

export class modalFirmaComponent implements AfterViewInit {

  //@ViewChild('signature') public signaturePad!: SignaturePadComponent;
  signaturePad!: SignaturePad;
  @ViewChild('canvas') canvasEl!: ElementRef;
  @ViewChild('division') divisionEl!: ElementRef;
  @Output() signImage: EventEmitter<string> = new EventEmitter();


  constructor(
    @Inject(MAT_DIALOG_DATA)
    public data: any,
    public dialogRef: MatDialogRef<modalFirmaComponent>
  ) {
    console.log(data);
  }


  async ngAfterViewInit(): Promise<void> {
    this.signaturePad = new SignaturePad(this.canvasEl.nativeElement);
    this.canvasResize();
  }


  /*async ngAfterViewInit(): Promise<void> {

    this.signaturePad.set('minWidth', 2); // set szimek/signature_pad options at runtime
    this.signaturePad.set('velocityFilterWeight', 0.7); // set szimek/signature_pad options at runtime
    this.signaturePad.set('minPointDistance', 0.5); // set szimek/signature_pad options at runtime
    this.signaturePad.set('throttle', 16); // set szimek/signature_pad options at runtime
    this.signaturePad.clear(); // invoke functions from szimek/signature_pad API
    this.canvasResize();
    this.signaturePad.fromData([]);
    const canvas = this.signaturePad.getCanvas();
    const ctx = canvas.getContext('2d');
    if(ctx) {
      ctx.fillStyle = 'black';
      const text = "FIRMA________________________________________________________";
      const x=20;
      const y= canvas.height-40;
      ctx.font = '10px Arial';
      ctx.fillStyle = 'black';
      ctx.fillText(text, x, y);
      // const backgroundImage = new Image();
      // backgroundImage.src = 'https://www.w3schools.com/html/img_chania.jpg';
      // backgroundImage.onload = () => {
      //   if (backgroundImage) {
      //     ctx.drawImage(backgroundImage, 0, 0, 500, 300);
      //   }
      // }
    }
  }*/

  canvasResize(){
    this.canvasEl.nativeElement.width = this.divisionEl.nativeElement.offsetWidth;
    this.canvasEl.nativeElement.height = this.divisionEl.nativeElement.offsetHeight;
  }

  /*canvasResize(): void {
    let canvas  = document.querySelector('canvas');
    if(canvas){
      canvas.width= window.innerWidth;
      canvas.height= window.innerHeight - (window.innerHeight * 0.19);
    }
  }*/

  onClear(): void {
    this.signaturePad.clear();
  }
  onCancelar(): void {
    this.dialogRef.close(0);
  }

  onGuardar(){
    const sign = this.signaturePad.toDataURL();
    const blob = this.dataURLToBlob(sign);
    const size = blob.size;
    const nombre = 'FirmaConformidad.jpg';

    this.dialogRef.close({blob, nombre, size});
    //this.dialogRef.close(sign);


    //console.log(blob);
  }

  dataURLToBlob(dataURL: string): Blob {
    const parts: string[] = dataURL.split(";base64,");
    const contentType: string = parts[0].split(":")[1];
    const raw: string = window.atob(parts[1]);
    const rawLength: number = raw.length;
    const uInt8Array: Uint8Array = new Uint8Array(rawLength);

    for (let i = 0; i < rawLength; ++i) {
      uInt8Array[i] = raw.charCodeAt(i);
    }

    return new Blob([uInt8Array], { type: contentType });
  }

  drawComplete(event: TouchEvent) {
    const sign = this.signaturePad.toDataURL();
    //console.log(sign);
    //this.signImage.emit(sign);
  }

  drawStart(event: TouchEvent) {
    //console.log('Start drawing', event);
  }

}
